from .gps_points_tools import Haversine, gps_point_range_check, simplified_float
from .time_format_functions import convert_to_erddap_time_format, time_zone_replace, time_to_str, str_to_timeobj, \
    str_to_timestamp, timestamp_format_check, time_comparison
from .list_compare import list_common_element
from .dataset_id_generate_function import IDGenerator



import collections


def compare_list2(list1, list2):
    """
    No Matter order if list1 and list2 contains same element then return True

    """
    a = list_check(list1)
    b = list_check(list2)
    res = collections.Counter(a) == collections.Counter(b)
    return res


def list_check(list_value):
    if type(list_value) is tuple:
        list_value = list(list_value)
    list_value = ["" if x is None else str(x) for x in list_value]

    return list_value


def compare_list(list1, list2):
    """
    To test two list are same or not if they are same then return True
    if they are not same return false but if list1 contain some value doesn't exist in
    list 2 then return the value exist in both list.
    """
    a = list_check(list1)
    b = list_check(list2)
    counter1 = collections.Counter(a)
    counter2 = collections.Counter(b)

    res1 = counter2 - counter1
    res2 = counter1 - counter2

    if res2:
        return list(counter1 - res2)
    else:
        if not res1:
            return True
        else:
            return False


def list_common_element(list1, list2, case_sensitive=False):
    """Given two lists and return a new list with common element of two list, the element comes from the first list
    """
    new_list = []

    for item1 in list1:
        for item2 in list2:
            if case_sensitive:
                if item1 == item2:
                    new_list.append(item1)
            else:

                if str(item1).lower() == str(item2).lower():
                    new_list.append(item1)
    return new_list



import datetime
import logging

logger = logging.getLogger(__name__)


class IDGenerator(object):
    GLIDER = 1

    @staticmethod
    def generate_id(project_id, **kwargs):
        """
        Generate dataset ID for ERDDAP, by given platform and other arguments
        :param project_id:
        :param args:
        :param kwargs:
        :return:
        """
        if project_id == IDGenerator.GLIDER:
            try:
                return IDGenerator.generate_glider_id(**kwargs)
            except KeyError as e:
                msg = "missing variable {}".format(e)
                logger.error(msg)
                return None
        else:
            msg = "Invalid platform type {}".format(project_id)
            raise ValueError(msg)

    @staticmethod
    def generate_glider_id(**kwargs):
        """
        Generate dataset id for glider platform
        :param platform_type:
        :param kwargs:
        :return:
        """
        platform_name = kwargs.pop("platform_name")
        # time_format_check
        start_time = kwargs.pop("start_time")
        deployment_number = kwargs.pop("deployment_number")
        test = kwargs.pop("test")
        start_time = datetime.datetime.strptime(start_time, '%Y-%m-%d %H:%M:%S')
        return IDGenerator._generate_glider_id(platform_name, start_time, deployment_number, test, **kwargs)

    @staticmethod
    def _generate_glider_id(platform_name, start_time, deployment_number, test, **kwargs):
        """
        Generate dataset id for slocum dataset id
        :param platform_name:
        :param start_time:
        :param deployment_number:
        :param test:
        :param kwargs:
        :return:
        """
        try:
            mode = kwargs.pop("mode")
        except ValueError:
            mode = False
            msg = "mode value is not given, default is delayed"
            logger.warning(msg)
        if mode:
            mode = 'realtime'
        else:
            mode = 'delayed'
        id_format = "{0}_{1:%Y%m%d}_{2}_{3}"
        if test:
            id_format = id_format + "_test"

        dataset_id = id_format.format(platform_name, start_time, deployment_number, mode)
        return dataset_id



import os


def check_create_dir(file_dir, recursively=False):
    """
    If dir exist just return the path, else create the dir and return the path
    """
    if not os.path.isdir(file_dir):
        if recursively:
            os.makedirs(file_dir)
        else:
            os.mkdir(file_dir)
    return file_dir



import math


# ------------------------------------------------------------------------------+
#
#   Nathan A. Rooy
#   Haversine Formula
#   June, 2016
#
# ------------------------------------------------------------------------------+

class Haversine:
    """
    use the haversine class to calculate the distance between
    two lon/lat coordnate pairs.
    output distance available in kilometers, meters, miles, and feet.
    example usage: Haversine([lon1,lat1],[lon2,lat2]).feet
    """

    def __init__(self):
        self.R = 6371000  # radius of Earth in meters

    def get_distance(self, coord1, coord2):
        lat1, lon1 = coord1
        lat2, lon2 = coord2
        phi_1 = math.radians(lat1)
        phi_2 = math.radians(lat2)

        delta_phi = math.radians(lat2 - lat1)
        delta_lambda = math.radians(lon2 - lon1)

        a = math.sin(delta_phi / 2.0) ** 2 + \
            math.cos(phi_1) * math.cos(phi_2) * \
            math.sin(delta_lambda / 2.0) ** 2
        c = 2 * math.atan2(math.sqrt(a), math.sqrt(1 - a))

        meters = self.R * c  # output distance in meters
        km = meters / 1000.0  # output distance in kilometers
        return km


def gps_point_range_check(point):
    lat = point[0]
    lng = point[1]
    if -90.0 < lat < 90.0 and -180.0 < lng < 180.0:
        return True
    return False


def simplified_float(number, keep_digit):
    """
    A function which can round the float (it is faster than round())

    """
    keep_digit = 10 ** keep_digit
    keep_digit = keep_digit / 100
    number = float(number)
    if number:
        number = number * keep_digit
        number = int(number)
        number = float(number) / keep_digit
    return number



import os


def is_sudo_user():
    """return True if script is triggered by a super user"""
    uid = os.getuid() if hasattr(os, 'getuid') else 65535
    gid = os.getgid() if hasattr(os, 'getgid') else 65535
    euid = os.geteuid() if hasattr(os, 'geteuid') else 65535
    egid = os.getegid() if hasattr(os, 'getegid') else 65535

    return not uid or not gid or not euid or not egid





"""You can find time relate function here"""

import time
import datetime
import logging

from pytz import timezone

logger = logging.getLogger(__file__)


def timeit(method):
    """
    credit to:
    https://medium.com/pythonhive/python-decorator-to-measure-the-execution-time-of-methods-fa04cb6bb36d
    Use as decorator to measure a function running time
    """

    def timed(*args, **kw):
        ts = time.time()
        result = method(*args, **kw)
        te = time.time()
        if 'log_time' in kw:
            name = kw.get('log_name', method.__name__.upper())
            kw['log_time'][name] = int((te - ts) * 1000)
        else:
            logger.debug('%r  %2.2f ms' % (method.__name__, (te - ts) * 1000))
        return result

    return timed


def time_zone_replace(time_str_or_time_object, time_zone='UTC'):
    """
    This function can convert time str to datetime object and replace its time zone to make Django happen
    enter None return None. default time zoom is UTC
    """
    if time_str_or_time_object:
        t = type(time_str_or_time_object)
        if t is str:
            et = datetime.datetime.strptime(time_str_or_time_object, "%Y-%m-%d %H:%M:%S")
        else:
            et = time_str_or_time_object
        time_str_or_time_object = et.replace(tzinfo=timezone(time_zone))
    return time_str_or_time_object


def time_to_str(time_str, time_str_format="%Y-%m-%d %H:%M:%S"):
    """Given time str return datetime object
    Given None return None
    Given datetime object return itself
    other raise"""
    res = None
    if time_str:
        t = type(time_str)
        if t is not str:
            str_time = time_str.strftime(time_str_format)
            res = str_time
        elif t == datetime:
            res = time_str
        else:
            msg = "Input must be string or datetime object"
            raise ValueError(msg)
    return res


def str_to_timeobj(time_str, time_format='%Y-%m-%d %H:%M:%S'):
    """Convert time str to datetime object"""
    try:
        time_obj = datetime.datetime.strptime(time_str, time_format)
    except Exception as e:
        if type(time_str) is datetime:
            return time_str
        msg = "Input must be string or datetime object\ninput: {}".format(time_str)
        raise ValueError(msg)

    return time_obj


def str_to_timestamp(time_str, time_format='%Y-%m-%d %H:%M:%S'):
    """Convert time string to Unix timestamp"""
    time_obj = str_to_timeobj(time_str, time_format=time_format)
    time_stamp = time_obj.timestamp()
    return time_stamp


def timestamp_format_check(time_stamp):
    # This is a bad solution
    if type(time_stamp) is str:
        time_stamp = float(time_stamp)

    if time_stamp > 9999999999:
        time_stamp = time_stamp / 1000.0

    return time_stamp


def time_comparison(first_time, second_time):
    if type(first_time) is float:
        first_time_obj = first_time
    else:
        first_time_obj = str_to_timeobj(first_time)
    if type(second_time) is float:
        second_time_obj = second_time
    else:
        second_time_obj = str_to_timeobj(second_time)
    if first_time_obj == second_time_obj:
        return 0
    if first_time_obj > second_time_obj:
        # first time is later than second time
        return 1
    if first_time_obj < second_time_obj:
        # first time is earlier than second time
        return -1


def convert_to_erddap_time_format(time_str):
    """
    accept time format like:
        2019-03-12 14:55:02
    convert it to
        2019-03-12T14:55:02Z

    if input was None, return None
    """
    ret_time_str = None
    if time_str and type(time_str) is str:
        ret_time_str = time_str.lstrip(' ')
        ret_time_str = ret_time_str.rstrip(' ')
        ret_time_str = ret_time_str.replace(" ", 'T')
        ret_time_str = ret_time_str + 'Z'
    return ret_time_str


def seconds_to_days_hours_minutes_seconds(seconds):
    """Given second and then convert it into day hour minutes and second"""
    days = 0
    hours = 0
    minutes = 0
    if seconds < 60:
        n_seconds = seconds
    else:
        minutes = int(seconds / 60)
        n_seconds = seconds - minutes * 60
        if minutes > 60:
            hours = int(minutes / 60)
            minutes = minutes - hours * 60
            if hours > 24:
                days = int(hours / 24)
                hours = hours - days * 24

    return days, hours, minutes, n_seconds


def time_period_overlap(start_time1, end_time1, start_time2, end_time2):
    """Check to see if time period 1 and time period 2 have overlap,
    return true if they have overlap and false they does not

    :param start_time1: datetime obj of start time of time period 1
    :param end_time1:  datetime obj of end time of time period 1
    :param start_time2: datetime obj of start time of time period 2
    :param end_time2: datetime obj of start time of time period 2
    :return: True means they have overlap and False means they have no overlap
    """

    def none_or_datetime(time_obj):
        if time_obj is None or type(time_obj) is datetime.datetime:
            return True
        return False

    if not (type(start_time1) is datetime.datetime and none_or_datetime(end_time1) and
            type(start_time2) is datetime.datetime and none_or_datetime(end_time2)):
        msg = "start_time must be datetime obj end_time must be datetime obj or None" \
              "\nstart_time1: {}\nend_time1: {}\nstart_time2: {}\nend_time2: {}".format(
            start_time1, end_time1, start_time2, end_time2)
        raise ValueError(msg)

    if end_time1 is not None and end_time2 is not None:
        if end_time1 <= start_time2:
            return False
        elif end_time2 <= start_time1:
            return False
        else:
            return True
    elif end_time1 is None and end_time2 is not None:
        if start_time1 >= end_time2:
            return False
    elif end_time1 is not None and end_time2 is None:
        if start_time2 >= end_time1:
            return False
    return True
